 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: lvgballse.c
 >>>>
 >>>>      Program Name: vgballse
 >>>>
 >>>> Date Last Updated: Tue Jul 28 16:40:37 1992 
 >>>>
 >>>>          Routines: lvgballse - the library call for vgballse
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vinclude.h"


/* -library_includes */
/* -library_includes_end */


/****************************************************************
*
* Routine Name: lvgballse - library call for vgballse
*
* Purpose:
*    
*    Creates a image of a ball with the origin at the center
*    
*    

* Input:
*    
*    image          input image structure
*    
*    type           the data storage type of the resulting image
*    
*    bgnd           the background intensity
*    
*    x              the X coordinate of the origin
*    
*    y              the Y coordinate of the origin
*    
*    origin         automatic or user-selected origin
*    
*    d              diameter of circle (in pixels)
*    
*    

* Output:
*    
*    image          holds the resulting output image.
*    
*    

*
* Written By: Greg Donohoe
*    
*    

****************************************************************/


/* -library_def */
int
lvgballse (image,  type, bgnd,  x, y, origin, d)
struct xvimage **image;
int type;
float bgnd,x,y,d;
int origin;
/* -library_def_end */

/* -library_code */
{
  int i,j;
  unsigned char *cpixel;
  int *ipixel;
  float r;
  struct xvimage *img, *createimage();
  int xorg, yorg;
  int rows, cols;
  float x_ctr, y_ctr;

  rows = d+2;
  cols = d+2;

  img   = createimage((unsigned long) rows,  /* number of rows */
                     (unsigned long) cols,   /* number of columns */
                     (unsigned long) type,   /* data storage type */
                     (unsigned long) 1,      /* num_of_images */
                     (unsigned long) 1,      /* num_data_bands */
                     "created by vgballse",    /* comment */
                     (unsigned long) 0,      /* map_row_size */
                     (unsigned long) 0,      /* map_col_size */
                     (unsigned long)
                     VFF_MS_NONE,            /* map_scheme */
                     (unsigned long)
                     VFF_MAPTYP_NONE,        /* map_storage_type */
                     (unsigned long)
                     VFF_LOC_IMPLICIT,       /* location_type */
                     (unsigned long) 0);     /* location_dim */


  if (img == NULL)
    {
      fprintf(stderr,"lvgballse: Unable to allocate new image!\n");
      *image = NULL;
      return(0);
    }

  *image = img;
  r = d/2.0;
  r = r*r;

  x_ctr = rows/2;
  y_ctr = cols/2;

  /* Set the SE origin:  either center it,  or accept user */
  /*  values */
  if (!origin) {
     xorg = (int)x_ctr;
     yorg = (int)y_ctr;
  }
  else {
     xorg = x;
     yorg = y;
  }


 img->ispare1 = -xorg;
 img->ispare2 = -yorg;

 x = (int)d/2.0;
 y = (int)d/2.0;

  switch(type)
    {
      case VFF_TYP_1_BYTE:
        cpixel = (unsigned char *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if (((i-y_ctr)*(i-y_ctr)+(j-x_ctr)*(j-x_ctr)) <= r)
                  *cpixel++ = 
                 (unsigned char)(sqrt(r - (i-y_ctr)*(i-y_ctr) - (j-x_ctr)*(j-x_ctr) + .5));
            else *cpixel++ = bgnd;
        break;
      case VFF_TYP_4_BYTE:
        ipixel = (int *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if ((i-y_ctr)*(i-y_ctr)+(j-x_ctr)*(j-x_ctr) <= r)
                  *ipixel++ = (int)(sqrt(r - (i-y_ctr)*(i-y_ctr) - (j-x_ctr)*(j-x_ctr) + .5));
            else *ipixel++ = bgnd;
        break;
      default:
        fprintf(stderr,"lvgballse: Unsupported data storage type requested\n");
        fprintf(stderr,"         Supported types: INT and BYTE\n");
        return(0);
    }

  return(1);
}
/* -library_code_end */
