 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: lvgdiskse.c
 >>>>
 >>>>      Program Name: vgdiskse
 >>>>
 >>>> Date Last Updated: Mon Jul 27 09:14:04 1992 
 >>>>
 >>>>          Routines: lvgdiskse - the library call for vgdiskse
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vinclude.h"


/* -library_includes */
/* -library_includes_end */


/****************************************************************
*
* Routine Name: lvgdiskse - library call for vgdiskse
*
* Purpose:
*    
*    Creates a image of a disk with the origin at the center
*    
*    

* Input:
*    
*    image          input image structure
*    
*    rows           the desired number of rows in output image
*    
*    cols           the desired number of columns in output image
*    
*    type           the data storage type of the resulting image
*    
*    bgnd           the background intensity
*    
*    fgnd           the forground intensity
*    
*    x              the X coordinate of the center of the circle
*    
*    y              the Y coordinate of the center of the circle
*    
*    d              diameter of circle (in pixels)
*    
*    

* Output:
*    
*    image          holds the resulting output image.
*    
*    

*
* Written By: Greg Donohoe
*    
*    

****************************************************************/


/* -library_def */
int
lvgdiskse (image, rows, cols, type, bgnd, fgnd, x, y, d)
struct xvimage **image;
int rows, cols, type;
float bgnd,fgnd,x,y,d;
/* -library_def_end */

/* -library_code */
{
  int i,j;
  unsigned char *c;
  float r;
  int *ipixel;
  struct xvimage *img, *createimage();
  int xorg, yorg;

  img   = createimage((unsigned long) rows,  /* number of rows */
                     (unsigned long) cols,   /* number of columns */
                     (unsigned long) type,   /* data storage type */
                     (unsigned long) 1,      /* num_of_images */
                     (unsigned long) 1,      /* num_data_bands */
                     "created by vgdiskse",    /* comment */
                     (unsigned long) 0,      /* map_row_size */
                     (unsigned long) 0,      /* map_col_size */
                     (unsigned long)
                     VFF_MS_NONE,            /* map_scheme */
                     (unsigned long)
                     VFF_MAPTYP_NONE,        /* map_storage_type */
                     (unsigned long)
                     VFF_LOC_IMPLICIT,       /* location_type */
                     (unsigned long) 0);     /* location_dim */


  if (img == NULL)
    {
      fprintf(stderr,"lvgdiskse: Unable to allocate new image!\n");
      *image = NULL;
      return(0);
    }

  *image = img;
  r = d/2.0;
  r = r*r;

  xorg = (int)d/2.0;
  yorg = (int)d/2.0;

 img->ispare1 = -xorg;
 img->ispare2 = -yorg;

  switch(type)
    {
      case VFF_TYP_1_BYTE:
        c = (unsigned char *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if ((i-y)*(i-y)+(j-x)*(j-x) <= r) *c++ = fgnd;
            else *c++ = bgnd;
        break;
      case VFF_TYP_4_BYTE:
        ipixel = (int *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if ((i-y)*(i-y)+(j-x)*(j-x) <= r) *ipixel++ = fgnd;
            else *ipixel++ = bgnd;
        break;
      default:
        fprintf(stderr,"lvgdiskse: Unsupported data storage type requested\n");
        fprintf(stderr,"         Supported types: INT and BYTE\n");
        return(0);
    }

  return(1);
}
/* -library_code_end */
