 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: lvgmerode.c
 >>>>
 >>>>      Program Name: vgmerode
 >>>>
 >>>> Date Last Updated: Thu Jul 23 16:04:58 1992 
 >>>>
 >>>>          Routines: lvgmerode - the library call for vgmerode
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vinclude.h"


/* -library_includes */
/* -library_includes_end */


/****************************************************************
*
* Routine Name: lvgmerode - library call for vgmerode
*
* Purpose:
*    
*    Erode a greyscale image X, by a structuring element B.
*    
*    

* Input:
*    
*    img            xvimage structure, must be a byte image.
*    
*    ker            xvimage structure, morphological kernel or  struc-
*                   turing  element.   Should  have  been  created  by
*                   vmcustom.
*    
*    

* Output:
*    
*    img2           xvimage structure, erosion of img1 by kernel ker.
*    
*    

*
* Written By: Pascal ADAM and Greg DONOHOE
*    
*    Has been updated to ghostX on 5/24/90 by Pascal ADAM
*    
*    

****************************************************************/


/* -library_def */
int
lvgmerode(image,kernel)
struct xvimage *image, *kernel;
/* -library_def_end */

/* -library_code */
{
    int belong,    /* TRAN(IMG2;i,j) belongs to IMG1 ? */ 
        nc,        /* number of columns of image */ 
        nr,        /*   "    "  rows    "  "    */
        starti,    /* x upper left corner position of kernel */
        startj,    /* y   "    "    "        "     "   "     */
        kernc,     /* number of columns of kernel */
        kernr;     /* number of rows of kernel */
    unsigned char *result; /* storage of final result */

    char   *program = "lvgmerode";

    int i,j,k,ii,jj,iii,jjj;
    register int minpix, imagepix,  kernpix;
    unsigned char *c1, *c3;
    int *c2;

    /* Check type,... of image */
    if (!(propertype(program, image, VFF_TYP_1_BYTE, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "lvgmerode: image must be of type int\n");
        return (0);
    }
    if (!(proper_num_images (program, image, 1, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, image, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

    nc = image->row_size; /* number of columns */
    nr = image->col_size; /* number of rows    */

    c1 = (unsigned char *)(image->imagedata); /* pointer on image's pixels */

    /* Check type,... of image */
    if (!(propertype(program, kernel, VFF_TYP_4_BYTE, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"lvgmerode: kernel must be of type int/n");
        return (0);
    }
    if (!(proper_num_images (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

    kernc = kernel->row_size;/* number of columns of the kernel */
    kernr = kernel->col_size;/* number of rows of the kernel    */

    starti = (int)kernel->ispare1; /* position of upper left hand */ 
    startj = (int)kernel->ispare2; /* corner of the kernel (x,y)  */

    c2 = (int *)(kernel->imagedata); /* pointer to kernel values */

    /* Checks if the relative position of the kernel relative to the 
    ** dimension of the image will give a significant result image  */
    if (starti>nr/2 || startj>nc/2) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Warning : the size of the kernel\n");
        (void) fprintf (stderr, "is too big compared to the image\n");
        (void) fprintf (stderr, "size. The visible result may out\n");
        (void) fprintf (stderr, "of the result image dimension...\n");
        }
   
    
    /* dynamic memory allocation for temporary result stockage */
    result = (unsigned char *) malloc( nc * nr *sizeof(unsigned char));
    if (result == NULL) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Insufficient Space avaliable\n");
        return(0);
    }

    c3 = result; /* pointer on the result image data */

    k = nc * nr; /* total number of pixels in the input image */
                                
    /* initialisation of the resulting image  */
    for (j=0;j<k;j++) *(c3 +j)=0;

    /* Scans  and computes the final value of the result image */
    for (i=0;i<nr;i++) {
      for (j=0;j<nc;j++) {

        /* Compute erosion.  Scan image and kernel.  For each non-negative */
        /* kernel location,  find difference between image and kernel.     */
        /* Replace center pixel with minimum of these difference,  clipped */
        /* to zero.                                                        */

        minpix = 255;           /* Initialize to max value */
        for (ii=0;  ii < kernr;  ii++) {
              iii = i + ii + starti;
              if (iii<0 || iii>=nr) continue;
              for (jj = 0;  jj < kernc; jj++) {
                    jjj = j + jj + startj;
                    if (jjj>=0 && jjj<nc) {
                       imagepix = *(c1+iii*nc+jjj);
                       kernpix = *(c2+ii*kernc+jj);
                       if (kernpix >= 0)
                          minpix = MIN(minpix, (imagepix - kernpix));
                    }
               }
            }
        if (minpix < 0)
           *(c3 + i*nc + j) = (unsigned char)0;
        else
           *(c3 + i*nc + j) = (unsigned char)minpix;
        
        }
      }
    
    c3=result;
    c1=(unsigned char *)image->imagedata;

    /* transfert result data in image data */
    for (i=0;i<k;i++) *(c1++) = *(c3++); 

    /* free memory location of temporary result storage */ 
    free(result);
    return(1);
}
/* -library_code_end */
