 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: lvgmopen.c
 >>>>
 >>>>      Program Name: vgmopen
 >>>>
 >>>> Date Last Updated: Sat Jul 25 11:37:06 1992 
 >>>>
 >>>>          Routines: lvgmopen - the library call for vgmopen
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vinclude.h"


/* -library_includes */
/* -library_includes_end */


/****************************************************************
*
* Routine Name: lvgmopen - library call for vgmopen
*
* Purpose:
*    
*    Open a greyscale image X, by a structuring element B.
*    
*    

* Input:
*    

* Output:
*    
*    img2           xvimage structure, opening of img1 by kernel ker.
*    
*    

*
* Written By: Pascal ADAM and Greg DONOHOE
*    
*    Has been updated to ghostX on 5/24/90 by Pascal ADAM
*    
*    

****************************************************************/


/* -library_def */
int
lvgmopen(image,kernel)
struct xvimage *image, *kernel;
/* -library_def_end */

/* -library_code */
{
    int belong,    /* TRAN(IMG2;i,j) belongs to IMG1 ? */ 
        nc,        /* number of columns of image */ 
        nr,        /*   "    "  rows    "  "    */
        starti,    /* x upper left corner position of kernel */
        startj,    /* y   "    "    "        "     "   "     */
        kernc,     /* number of columns of kernel */
        kernr;     /* number of rows of kernel */
        unsigned char *result; /* storage of final result */

    char   *program = "lvgmopen";

    int i,j,k,ii,jj,iii,jjj;
    register int minpix, maxpix, imagepix,  kernpix;
    unsigned char *c1, *c3;
    int *c2;

    /* Check type,... of image */
    if (!(propertype(program, image, VFF_TYP_1_BYTE, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "lvgmopen: image must be of type byte\n");
        return (0);
    }
    if (!(proper_num_images (program, image, 1, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, image, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

    nc = image->row_size; /* number of columns */
    nr = image->col_size; /* number of rows    */

    c1 = (unsigned char *)(image->imagedata); /* pointer on image's pixels */

    /* Check type,... of image */
    if (!(propertype(program, kernel, VFF_TYP_4_BYTE, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"lvgmopen: kernel must be of type int\n");
        return (0);
    }
    if (!(proper_num_images (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

    kernc = kernel->row_size;/* number of columns of the kernel */
    kernr = kernel->col_size;/* number of rows of the kernel    */

    starti = (int)kernel->ispare1; /* position of upper left hand */ 
    startj = (int)kernel->ispare2; /* corner of the kernel (x,y)  */

    c2 = (int *)(kernel->imagedata); /* pointer on kernel values */

    /* Checks if the relative position of the kernel relative to the 
    ** dimension of the image will give a significant result image  */
    if (starti>nr/2 || startj>nc/2) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Warning : the size of the kernel\n");
        (void) fprintf (stderr, "is too big compared to the image\n");
        (void) fprintf (stderr, "size. The visible result may out\n");
        (void) fprintf (stderr, "of the result image dimension...\n");
        }
   
    
    /* dynamic memory allocation for temporary result stockage */
    result = (unsigned char *) malloc( nc * nr * sizeof(int));
    if (result == NULL) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Insufficient Space avaliable\n");
        return(0);
    }

    c3 = result; /* pointer on the result image data */

    k = nc * nr; /* total number of pixels in the input image */
                                
    /* initialisation of the resulting image  */
    for (j=0;j<k;j++) *(c3 +j)=0;

    /* Scans  and computes the final value of the result image */
    
    /* First ERODE the image by the structuring element */
    for (i=0;i<nr;i++) {
      for (j=0;j<nc;j++) {

           minpix = 255; 
           for (ii=0;  ii < kernr;  ii++) {
                 iii = i + ii + starti;
                 if (iii<0 || iii>=nr) continue;
                 for (jj = 0;  jj < kernc; jj++) {
                       jjj = j + jj + startj;
                       if (jjj>=0 && jjj<nc ) {
                          imagepix = *(c1+iii*nc+jjj);
                          kernpix = *(c2+ii*kernc+jj);
                          if (kernpix >= 0)
                             minpix = MIN(minpix, (imagepix - kernpix));
                        }
                    }
               }
           minpix = MAX(minpix, 0);
           *(c3 + i*nc + j) = (unsigned char)minpix;
        
           }
         }
    

    /* Now DILATE the image by the structuring element */
    /*  Use the input image for the result */
    for (i=0;i<nr;i++) {
      for (j=0;j<nc;j++) {
           maxpix = 0;           
           for (ii=0;  ii < kernr;  ii++) {
                 iii = i + ii + starti;
                 if (iii<0 || iii>nr) continue;
                 for (jj = 0;  jj < kernc; jj++) {
                       jjj = j + jj + startj;
                       if (jjj>=0 && jjj<nc) {
                          imagepix = *(c3+iii*nc+jjj);
                          kernpix = *(c2+ii*kernc+jj);
                          if (kernpix >= 0)
                             maxpix = MAX(maxpix, (imagepix + kernpix));
                        }
                    }
               }
           maxpix = MIN(maxpix, 255);
           *(c1 + i*nc + j) = (unsigned char)maxpix;
        }
      }


    /*  Now the input image containes the opening */

    /* free memory location of temporary result storage */ 
    free(result);
    return(1);
}
/* -library_code_end */
