 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: lvgmtophat.c
 >>>>
 >>>>      Program Name: vgmtophat
 >>>>
 >>>> Date Last Updated: Mon Jul 27 16:52:02 1992 
 >>>>
 >>>>          Routines: lvgmtophat - the library call for vgmtophat
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vinclude.h"


/* -library_includes */
#include <limits.h>
/* -library_includes_end */


/****************************************************************
*
* Routine Name: lvgmtophat - library call for vgmtophat
*
* Purpose:
*    
*    Perform a tophat transform on image  X,   using  structuring
*    element B.
*    
*    

* Input:
*    
*    image          xvimage structure, must be a byte image.
*    
*    kernel         xvimage structure, morphological kernel or  struc-
*                   turing  element.   Should  have  been  created  by
*                   vmcustom.
*    
*    s_float        scale factor:  proportion  of  smoothed  image  to
*                   subtract
*    
*    normal_logical normalize output image?
*    
*    

* Output:
*    
*    img2           xvimage structure, opening of img1 by kernel ker.
*    
*    

*
* Written By: Greg DONOHOE and Pascal ADAM
*    
*    Has been updated to ghostX on 5/24/90 by Pascal ADAM
*    
*    

****************************************************************/


/* -library_def */
int
lvgmtophat(image1,  kernel, scale, normalize)
struct xvimage *image1,  *kernel;
float scale;
int normalize;
/* -library_def_end */

/* -library_code */
{
    int belong,    /* TRAN(IMG2;i,j) belongs to IMG1 ? */ 
        nc,        /* number of columns of image */ 
        nr,        /*   "    "  rows    "  "    */
        starti,    /* x upper left corner position of kernel */
        startj,    /* y   "    "    "        "     "   "     */
        kernc,     /* number of columns of kernel */
        kernr;     /* number of rows of kernel */
        int *result; /* storage of final result */

    char   *program = "lvgmtophat";

    int i,j,k,ii,jj,iii,jjj;
    register int minpix, maxpix, imagepix,  kernpix, index;
    struct xvimage *img, *createimage();
    unsigned char *c1;
    int *c2, *c3;
    int *c4;
    float fmax, fmin;   /* max and min for normalization */
    float a, b;         /* scale factor and offset for normlzn */

    /* Check type,... of image */
    if (!(propertype(program, image1, VFF_TYP_1_BYTE, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "lvgmtophat: image must be of type byte\n");
        return (0);
    }
    if (!(proper_num_images (program, image1, 1, FALSE))) {
        (void) fprintf (stderr, "\n\n%s:   ", program);
        (void) fprintf (stderr, "Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, image1, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

    nc = image1->row_size; /* number of columns */
    nr = image1->col_size; /* number of rows    */

    c1 = (unsigned char *)(image1->imagedata); /* pointer on image's pixels */

    /* Check type,... of image */
    if (!(propertype(program, kernel, VFF_TYP_4_BYTE, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"lvgmtophat: kernel must be of type int\n");
        return (0);
    }
    if (!(proper_num_images (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on files with one image\n\n");
        return (0);
    }
    if (!(proper_num_bands (program, kernel, 1, FALSE))) {
        (void) fprintf (stderr,"\n\n%s:   ", program);
        (void) fprintf (stderr,"Can only work on images with 1 data band\n\n");
        return (0);
    }

 img   =   createimage((unsigned long) nr,  /* number of rows */
                     (unsigned long) nc,   /* number of columns */
                     (unsigned long)       /* data storage type */
                     VFF_TYP_4_BYTE,
                     (unsigned long) 1,      /* num_of_images */
                     (unsigned long) 1,      /* num_data_bands */
                     "created by vgmtophat",    /* comment */
                     (unsigned long) 0,      /* map_row_size */
                     (unsigned long) 0,      /* map_col_size */
                     (unsigned long)
                     VFF_MS_NONE,            /* map_scheme */
                     (unsigned long)
                     VFF_MAPTYP_NONE,        /* map_storage_type */
                     (unsigned long)
                     VFF_LOC_IMPLICIT,       /* location_type */
                     (unsigned long) 0);     /* location_dim */


    img;

    kernc = kernel->row_size;/* number of columns of the kernel */
    kernr = kernel->col_size;/* number of rows of the kernel    */

    starti = (int)kernel->ispare1; /* position of upper left hand */ 
    startj = (int)kernel->ispare2; /* corner of the kernel (x,y)  */

    c2 = (int *)(kernel->imagedata); /* pointer on kernel values */

    /* Checks if the relative position of the kernel relative to the 
    ** dimension of the image will give a significant result image  */
    if (starti>nr/2 || startj>nc/2) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Warning : the size of the kernel\n");
        (void) fprintf (stderr, "is too big compared to the image\n");
        (void) fprintf (stderr, "size. The visible result may out\n");
        (void) fprintf (stderr, "of the result image dimension...\n");
        }
   
    
    /* dynamic memory allocation for temporary result stockage */
    result = (int *) malloc( nc * nr *sizeof(unsigned int));
    if (result == NULL) {
        (void) fprintf (stderr, "\n%s:   ", program);
        (void) fprintf (stderr, "Insufficient Space avaliable\n");
        return(0);
    }

    c3 = result; /* pointer on the result image data */

    k = nc * nr; /* total number of pixels in the input image */
                                
    /* initialisation of the resulting image  */
    for (j=0;j<k;j++) *(c3 +j)=0;

    /* Scans  and computes the final value of the result image */
    
    /* First ERODE the image by the structuring element */
    for (i=0;i<nr;i++) {
      for (j=0;j<nc;j++) {

           minpix = 255; 
           for (ii=0;  ii < kernr;  ii++) {
                 iii = i + ii + starti;
                 for (jj = 0;  jj < kernc; jj++) {
                       jjj = j + jj + startj;
                       imagepix = *(c1+iii*nc+jjj);
                       kernpix = *(c2+ii*kernc+jj);
                       if (kernpix != 0)
                          minpix = MIN(minpix, (imagepix - kernpix));
                    }
               }
/*
           Remove the clipping for intermediate result
           minpix = MAX(minpix, 0);
*/
           *(c3 + i*nc + j) = (int)minpix;
        
           }
         }
    
     c4 = (int *)(img->imagedata);
     fmin = INT_MAX;
     fmax = INT_MIN;

    /* Now DILATE the image by the structuring element */
    /*  Use the input image for the result */
    for (i=0;i<nr;i++) {
      for (j=0;j<nc;j++) {
           maxpix = 0;           
           for (ii=0;  ii < kernr;  ii++) {
                 iii = i + ii + starti;
                 if (iii<0 || iii>=nr) continue;
                 for (jj = 0;  jj < kernc; jj++) {
                       jjj = j + jj + startj;
                       if (jjj >=0 && jjj<nc) {
                          imagepix = *(c3+iii*nc+jjj);
                          kernpix = *(c2+ii*kernc+jj);
                          if (kernpix >= 0)
                          maxpix = MAX(maxpix, (imagepix + kernpix));
                       }
                    }
               }
           index = i*nc + j;
           imagepix = *(c1 + index) - (int)(scale*(float)maxpix);
           imagepix = MAX(imagepix, 0);
           fmax = MAX(imagepix, fmax);
           fmin = MIN(imagepix, fmin);
           *(c4 + index) = (unsigned char)(imagepix);
        }
      }


    /*  Copy the integer result to the byte-valued input image.  */
    /*  If the 'normalize' logical variable is set,  normalize   */
    /*  the output image;  otherwise,  just clip it.             */

    c1 = (unsigned char *)(image1->imagedata);
    c4 = (int *)(img->imagedata);

    if (normalize) {
          a = (float)255/(fmax-fmin);
          b = - a*fmin;
          for (i=0; i<k; i++) {
                imagepix = (int)(a*(float)c4[i] + b);
                c1[i] = (unsigned char)imagepix;
             }
       }
    else {
          for (i=0; i<k; i++)
             c1[i] = (unsigned char ) MIN((c4[i]),255);
       }

    /* free memory location of temporary result storage */ 
    free(result);
    return(1);
}
/* -library_code_end */
